package com.example.demos.web;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.*;

/**
 * @author DongXiaoDong
 * @version 1.0
 * @date 2024/3/20 9:47
 * @description
 */
public class Service {
    /**
     * 线程池优化
     */
    private final ExecutorService executorService;

    /**
     * 构造函数，初始化线程池
     */
    public Service() {
        this.executorService = Executors.newFixedThreadPool(1);
    }

    /**
     * 优化后的线程池
     * @param userIds
     * @return
     */
    public Map<Long, String> get(List<Long> userIds) {
        Map<Long, String> resultMap = new ConcurrentHashMap<>();
        List<List<Long>> batches = splitIntoBatches(userIds, 1000);


        List<Callable<Map<Long, String>>> tasks = new ArrayList<>();
        for (List<Long> batch : batches) {
            tasks.add(() -> UserService.getUserMap(batch));
        }

        try {
            List<Future<Map<Long, String>>> futures = executorService.invokeAll(tasks);

            for (Future<Map<Long, String>> future : futures) {
                resultMap.putAll(future.get());
            }
        } catch (InterruptedException | ExecutionException e) {
            e.printStackTrace();
        }

        executorService.shutdown();

        return resultMap;
    }

    /**
     * 分割成小批次
     * @param userList
     * @param batchSize
     * @return
     */
    private List<List<Long>> splitIntoBatches(List<Long> userList, int batchSize) {
        List<List<Long>> batches = new ArrayList<>();
        int size = userList.size();

        for (int i = 0; i < size; i += batchSize) {
            if (i + batchSize > size) {
                batches.add(userList.subList(i, size));
            } else {
                batches.add(userList.subList(i, i + batchSize));
            }
        }

        return batches;
    }
}
